@@ -34,10 +34,10 @@ HEADER_FILE = """
 #      username: alice
 #      password: "secret password"
 #    assets:
-#      - id: 12345
-#        config:
-#          username: bob
-#          password: "my secret"
+#    - id: 12345
+#      config:
+#        username: bob
+#        password: "my secret"
 """.lstrip()
 
 AGENTCORE_HOST = os.getenv('AGENTCORE_HOST', '127.0.0.1')
@@ -56,6 +56,12 @@ ASSET_NAME_IDX, CHECK_NAME_IDX = range(2)
 # from reading a passwords directly from open configuration files.
 FERNET = Fernet(b"4DFfx9LZBPvwvCpwmsVGT_HzjgiGUHduP1kq_L2Fbjw=")
 
+MAX_PACKAGE_SIZE = int(os.getenv('MAX_PACKAGE_SIZE', 500))
+if 1 > MAX_PACKAGE_SIZE > 2000:
+    sys.exit('Value for MAX_PACKAGE_SIZE must be between 1 and 2000')
+
+MAX_PACKAGE_SIZE *= 1000
+
 
 class Probe:
     """This class should only be initialized once."""
@@ -193,8 +199,13 @@ class Probe:
             data=[path, check_data]
         )
 
-        if self._protocol and self._protocol.transport:
-            self._protocol.transport.write(pkg.to_bytes())
+        data = pkg.to_bytes()
+        if len(data) > MAX_PACKAGE_SIZE:
+            e = CheckException(f'data package too large ({len(data)} bytes)')
+            logging.error(f'check error; asset_id `{asset_id}`; {str(e)}')
+            self.send(path, None, e.to_dict(), ts)
+        elif self._protocol and self._protocol.transport:
+            self._protocol.transport.write(data)
 
     def close(self):
         if self._protocol and self._protocol.transport:
@@ -210,12 +221,13 @@ class Probe:
 
         if config:
             # First encrypt everything
-            encrypt(config, FERNET)
+            changed = encrypt(config, FERNET)
 
             # Re-write the file
-            with open(self._config_path, 'w') as file:
-                file.write(HEADER_FILE)
-                file.write(yaml.dump(config))
+            if changed:
+                with open(self._config_path, 'w') as file:
+                    file.write(HEADER_FILE)
+                    file.write(yaml.dump(config))
 
             # Now decrypt everything so we can use the configuration
             decrypt(config, FERNET)
@@ -232,13 +244,13 @@ class Probe:
         self._local_config_mtime = self._config_path.stat().st_mtime
         self._local_config = config
 
-    def _asset_config(self, asset_id: int) -> dict:
+    def _asset_config(self, asset_id: int, use: Optional[str]) -> dict:
         try:
             self._read_local_config()
         except Exception:
             logging.warning('new config file invalid, keep using previous')
 
-        return get_config(self._local_config, self.name, asset_id)
+        return get_config(self._local_config, self.name, asset_id, use)
 
     def _on_unset_assets(self, asset_ids: list):
         asset_ids = set(asset_ids)
@@ -318,7 +330,6 @@ class Probe:
                 break
             ts = ts_next
 
-            asset_config = self._asset_config(asset.id)
             (asset_name, _), config = self._checks_config[path]
             interval = config.get('_interval')
             timeout = 0.8 * interval
@@ -326,6 +337,8 @@ class Probe:
                 # asset_id and check_key are truly immutable, name is not
                 asset = Asset(asset_id, asset_name, check_key)
 
+            asset_config = self._asset_config(asset.id, config.get('_use'))
+
             logging.debug(f'run check; {asset}')
 
             try:
